#ifndef SOCKET_H
#define SOCKET_H

#include <string>

using namespace std;

#include <sys/socket.h>
#include <netinet/in.h>

int lookup_service(const string &name);

class InAddr;

class InAddrList {
  InAddr **addrs;
  int n_addrs;
 public:
  InAddrList();
  void add(InAddr *);
  InAddr *get(int i) { return addrs[i]; }
  int size() { return n_addrs; }
  ~InAddrList();
};

class InAddr {
public:
  static InAddrList *resolv(const char *name);
  static InAddr *create(const struct sockaddr *addr);

  virtual ~InAddr() { };
  virtual void set_port(int port)=0;
  virtual string tostring() = 0;
  virtual sockaddr * addr()=0;
  virtual socklen_t addr_len() = 0;
  virtual int af()=0;

  virtual operator sockaddr* () { return addr(); };
};

class Socket {
  int fd;
  struct sockaddr_in address;
  bool dead;
  string buffer;
 public:
  bool getdead() { return dead; }
  Socket();
  int connect(InAddr *where);
  int read(char *, int);
  int write(const char *, int);
  void close();
  virtual ~Socket();
  int getfd() { return fd; }
};

struct url {
  string protocol;
  string hostname;
  string service;

  url(const char *);
};

#endif
